pub trait Transformable {
// Required method
fn transform(&mut self, matrix: &Mat4);
}Expand description
A trait for types that can have a matrix transform applied. Primarily for testing/examples.
pub trait Transformable {
// Required method
fn transform(&mut self, matrix: &Mat4);
}A trait for types that can have a matrix transform applied. Primarily for testing/examples.